@@ -111,6 +111,8 @@ if PY3:
             raise exc.with_traceback(tb)
         raise exc
 
+    import pickle as cPickle
+
 else:
     import __builtin__ as builtins
     import copy_reg as copyreg
@@ -303,6 +305,9 @@ else:
         pass
 
 
+    import cPickle
+
+
 def getargspec(func):
     """Version of inspect.getargspec that works with partial and warps."""
     if isinstance(func, functools.partial):
