@@ -247,15 +247,6 @@ else:
 concat_dispatch = Dispatch('concat')
 
 
-@concat_dispatch.register_lazy('cudf')
-def register_cudf():
-    import cudf
-    concat_dispatch.register(
-        (cudf.DataFrame, cudf.Series, cudf.Index),
-        cudf.concat
-    )
-
-
 def concat(dfs, axis=0, join='outer', uniform=False, filter_warning=True):
     """Concatenate, handling some edge cases:
 
@@ -392,18 +383,3 @@ def concat_pandas(dfs, axis=0, join='outer', uniform=False, filter_warning=True)
     if ind is not None:
         out.index = ind
     return out
-
-
-def merge(left, right, how, left_on, right_on,
-          left_index, right_index, indicator, suffixes,
-          default_left, default_right):
-
-    if not len(left):
-        left = default_left
-
-    if not len(right):
-        right = default_right
-
-    return pd.merge(left, right, how=how, left_on=left_on, right_on=right_on,
-                    left_index=left_index, right_index=right_index,
-                    suffixes=suffixes, indicator=indicator)
